//
//  AMEvents.m
//  AMFatFractal
//
//  Created by Andre Muis on 8/25/13.
//  Copyright (c) 2013 Andre Muis. All rights reserved.
//

#import "AMEvents.h"

#import "AMEvent.h"

@interface AMEvents ()

@property (strong, nonatomic, readonly) NSTimer *timer;
@property (strong, nonatomic, readonly) NSMutableArray *events;

@end

@implementation AMEvents

- (id)init
{
    self = [super init];
    
    if (self) 
    {
        _timer = [NSTimer timerWithTimeInterval: 0.01
                                         target: self
                                       selector: @selector(timerFired:)
                                       userInfo: nil
                                        repeats: YES];
   
        [[NSRunLoop currentRunLoop] addTimer: self.timer
                                     forMode: NSDefaultRunLoopMode];
        
        _events = [[NSMutableArray alloc] init];
    }
    
    return self;
}

- (void)addEvent: (AMEvent *)event
{
    if ([self hasEvent: event] == NO)
    {
        [self.events addObject: event];
    }
}

- (BOOL)hasEvent: (AMEvent *)event
{
    BOOL exists = NO;
    
    for (AMEvent *someEvent in self.events)
    {
        if (someEvent.target == event.target && someEvent.action == event.action)
        {
            exists = YES;
        }
    }

    return exists;
}

- (void)removeEvent: (AMEvent *)event
{
    if ([self.events indexOfObject: event] != NSNotFound)
    {    
        [self.events removeObject: event];
    }
}

- (void)timerFired: (NSTimer *)timer
{
    for (AMEvent *event in [NSArray arrayWithArray: self.events])
    {
        if (event.elapsedSecs >= event.intervalSecs)
        {
            event.elapsedSecs = 0.0;
            
            if ([event.target respondsToSelector: event.action] == YES)
            {
                #pragma clang diagnostic push
                #pragma clang diagnostic ignored "-Warc-performSelector-leaks"
                [event.target performSelector: event.action withObject: event.object];
                #pragma clang diagnostic pop
            }
            else
            {
                NSLog(@"Target: %@ does not respond to selector: %@", event.target, NSStringFromSelector(event.action));
            }

            if (event.repeats == NO)
            {
                [self removeEvent: event];
            }
        }

        event.elapsedSecs = event.elapsedSecs + 0.01;
    }
}

@end














