//
//  AMFatFractal.m
//  AMFatFractal
//
//  Created by Andre Muis on 8/25/13.
//  Copyright (c) 2013 Andre Muis. All rights reserved.
//

#import "AMFatFractal.h"

#import "AMCreateRequest.h"
#import "AMDeleteRequest.h"
#import "AMGetArrayRequest.h"
#import "AMGetRequest.h"
#import "AMLoginRequest.h"
#import "AMRequestQueue.h"
#import "AMUpdateRequest.h"

@interface AMFatFractal ()

@property (strong, nonatomic, readonly) AMRequestQueue *requestQueue;

@end

@implementation AMFatFractal

- (id)initWithBaseSSLURL: (NSString *)baseSSLURL
       retryDelaySeconds: (float)retryDelaySeconds
{
    self = [super initWithBaseUrl: baseSSLURL];
    
    if (self)
    {
        _requestQueue = [[AMRequestQueue alloc] initWithAMFatFractal: self retryDelaySeconds: retryDelaySeconds];
    }
    
    return self;
}

- (int)queuedRequestsCount
{
    return self.requestQueue.count;
}

- (AMRequest *)amLoginWithUserName: (NSString *)userName
                       andPassword: (NSString *)password
                 completionHandler: (FFHttpMethodCompletion)completionHandler
{
    AMLoginRequest *loginRequest = [[AMLoginRequest alloc] initWithUserName: userName
                                                                   password: password
                                                          completionHandler: completionHandler];
    
    [self.requestQueue add: loginRequest];
    [self.requestQueue process];
    
    return loginRequest;
}

- (AMRequest *)amGetObjectFromURI: (NSString *)uri
                completionHandler: (FFHttpMethodCompletion)completionHandler
{
    AMGetRequest *getRequest = [[AMGetRequest alloc] initWithURI: uri
                                               completionHandler: completionHandler];
    
    [self.requestQueue add: getRequest];
    [self.requestQueue process];
    
    return getRequest;
}

- (AMRequest *)amGetArrayFromURI: (NSString *)uri
               completionHandler: (FFHttpMethodCompletion)completionHandler
{
    AMGetArrayRequest *getArrayRequest = [[AMGetArrayRequest alloc] initWithURI: uri
                                                              completionHandler: completionHandler];
    
    [self.requestQueue add: getArrayRequest];
    [self.requestQueue process];
    
    return getArrayRequest;
}

- (AMRequest *)amCreateObject: (id)object
                        atURI: (NSString *)uri
            completionHandler: (FFHttpMethodCompletion)completionHandler
{
    AMCreateRequest *createRequest = [[AMCreateRequest alloc] initWithObject: object
                                                                         uri: uri
                                                           completionHandler: completionHandler];

    [self.requestQueue add: createRequest];
    [self.requestQueue process];

    return createRequest;
}

- (AMRequest *)amUpdateObject: (id)object
            completionHandler: (FFHttpMethodCompletion)completionHandler
{
    AMUpdateRequest *updateRequest = [[AMUpdateRequest alloc] initWithObject: object
                                                           completionHandler: completionHandler];
    
    [self.requestQueue add: updateRequest];
    [self.requestQueue process];
    
    return updateRequest;
}

- (AMRequest *)amDeleteObject: (id)object
            completionHandler: (FFHttpMethodCompletion)completionHandler
{
    AMDeleteRequest *deleteRequest = [[AMDeleteRequest alloc] initWithObject: object
                                                           completionHandler: completionHandler];
    
    [self.requestQueue add: deleteRequest];
    [self.requestQueue process];
    
    return deleteRequest;
}

- (void)amCancelRequest: (AMRequest *)request
{
    [self.requestQueue cancel: request];
}

@end
















