//
//  AMFatFractalTests.m
//  AMFatFractalTests
//
//  Created by Andre Muis on 8/25/13.
//  Copyright (c) 2013 Andre Muis. All rights reserved.
//

// Class under test
#import "AMFatFractal.h"

// Collaborators
#import "AMRequest.h"   

// Test support
#import <SenTestingKit/SenTestingKit.h>

// Uncomment the next two lines to use OCHamcrest for test assertions:
#define HC_SHORTHAND
#import <OCHamcrestiOS/OCHamcrestIOS.h>

@interface ObjectForTesting : NSObject
@end

@implementation ObjectForTesting
@end

@interface AMFatFractalTests : SenTestCase

@property (assign, nonatomic) BOOL done;
@property (strong, nonatomic, readonly) AMFatFractal *amFatFractal;
@property (strong, nonatomic) ObjectForTesting *objectForTesting;

@end

@implementation AMFatFractalTests

- (void)setUp
{
    [super setUp];
    
    self.done = NO;

    self.objectForTesting = [[ObjectForTesting alloc] init];

    NSString *baseSSLURL = @"https://localhost:8443/mazes";
    
    _amFatFractal = [[AMFatFractal alloc] initWithBaseSSLURL: baseSSLURL
                                           retryDelaySeconds: 5.0];
    
    [self.amFatFractal amLoginWithUserName: @"system"
                               andPassword: @"password"
                         completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
     {
         assertThat(theErr, nilValue());
         assertThatInt(theResponse.statusCode, equalToInt(200));
         
         self.done = YES;
     }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));

    self.done = NO;
}

- (void)tearDown
{
    self.done = NO;
    _amFatFractal = nil;
    
    [super tearDown];
}

- (void)testCreate
{
    // given

    // when
    [self.amFatFractal amCreateObject: self.objectForTesting
                                atURI: @"/ObjectForTesting"
                    completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
    {
        // then
        assertThat(theErr, nilValue());
        assertThatInt(theResponse.statusCode, equalToInt(201));
         
        self.done = YES;
    }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));
}

- (void)testGet
{
    // given
    
    // when
    [self.amFatFractal amGetObjectFromURI: @"/ObjectForTesting?count=1&start=0"
                        completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
    {
        // then
        assertThat(theErr, nilValue());
        assertThatInt(theResponse.statusCode, equalToInt(200));
         
        self.done = YES;
    }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));
}

- (void)testGetArray
{
    // given
    
    // when
    [self.amFatFractal amGetArrayFromURI: @"/ObjectForTesting"
                       completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
    {
        // then
        assertThat(theErr, nilValue());
        assertThatInt(theResponse.statusCode, equalToInt(200));
         
        self.done = YES;
    }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));
}

- (void)testUpdate
{
    // given
    [self.amFatFractal amCreateObject: self.objectForTesting
                                atURI: @"/ObjectForTesting"
                    completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
     {
         // then
         assertThat(theErr, nilValue());
         assertThatInt(theResponse.statusCode, equalToInt(201));
         
         self.done = YES;
     }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));

    self.done = NO;

    // when
    [self.amFatFractal amUpdateObject: self.objectForTesting
                    completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
    {
        // then
        assertThat(theErr, nilValue());
        assertThatInt(theResponse.statusCode, equalToInt(200));
         
        self.done = YES;
    }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));
}

- (void)testDelete
{
    // given
    [self.amFatFractal amCreateObject: self.objectForTesting
                                atURI: @"/ObjectForTesting"
                    completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
     {
         // then
         assertThat(theErr, nilValue());
         assertThatInt(theResponse.statusCode, equalToInt(201));
         
         self.done = YES;
     }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));
    
    self.done = NO;
    
    // when
    [self.amFatFractal amDeleteObject: self.objectForTesting
                    completionHandler: ^(NSError *theErr, id theObj, NSHTTPURLResponse *theResponse)
    {
        // then
        assertThat(theErr, nilValue());
        assertThatInt(theResponse.statusCode, equalToInt(200));
         
        self.done = YES;
    }];
    assertThatBool([self waitForCompletion: 15.0 * 60.0], is(equalToBool(YES)));
}

// assertThatBool([self waitForCompletion:90.0], is(equalToBool(YES)));
- (BOOL)waitForCompletion: (NSTimeInterval)timeoutSecs
{
    NSDate *timeoutDate = [NSDate dateWithTimeIntervalSinceNow: timeoutSecs];
    
    do
    {
        [[NSRunLoop currentRunLoop] runMode: NSDefaultRunLoopMode beforeDate: timeoutDate];
        
        if ([timeoutDate timeIntervalSinceNow] < 0.0)
        {
            break;
        }
    } while (!self.done);
    
    return self.done;
}

@end




















